defmodule Chronoscope.Gemini.Response do
  @response_pattern ~r/^(?<status_code>[0-9]{2}) (?<meta>.+)\r\n(?<body>(?s:.)*)/

  def parse(response) do
    response
    |> to_string()
    |> parse_response()
  end

  defp parse_response(response) do
    case Regex.named_captures(@response_pattern, response) do
      nil ->
        {:error, "bad response: #{response}"}

      match ->
        match
        |> Map.update!("status_code", &String.to_integer/1)
        |> to_atom_keys()
        |> then(&{:ok, &1})
    end
  end

  defp to_atom_keys(response) do
    Map.new(response, fn {k, v} -> {String.to_atom(k), v} end)
  end
end
