defmodule Chronoscope.Gemini.ResponseTest do
  use Chronoscope.Case, async: true

  import Chronoscope.Gemini.Response

  describe "Chronoscope.Gemini.Response.parse()" do
    test "handles an empty response" do
      assert parse([]) == {:error, "bad response: "}
    end

    test "handles an bad http response" do
      response =
        ("HTTP/1.1 400 Bad Request\r\nServer: nginx\r\nDate: Tue, 07 May 2024 14:03:56 GMT\r\nContent-Type: text/html\r\nContent-Length: 150\r\nConnection: close\r\n" <>
           "\r\n<html>\r\n<head><title>400 Bad Request</title></head>\r\n<body>\r\n<center><h1>400 Bad Request</h1></center>\r\n<hr><center>nginx</center>\r\n</body>\r\n</html>\r\n")
        |> String.to_charlist()

      assert parse(response) == {:error, "bad response: " <> to_string(response)}
    end

    test "handles a good response" do
      response = "20 text/gemini\r\nHello, world!\r\n" |> String.to_charlist()

      assert parse(response) == {:ok, %{body: "Hello, world!\r\n", meta: "text/gemini", status_code: 20}}
    end

    test "handles a redirect" do
      response = "31 gemini://localhost/grapes\r\n" |> String.to_charlist()

      assert parse(response) == {:ok, %{body: "", meta: "gemini://localhost/grapes", status_code: 31}}
    end
  end
end
